"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.EmojiPicker = void 0;

var _react = _interopRequireWildcard(require("react"));

var _utils = require("./utils");

var _Navbar = _interopRequireDefault(require("./EmojiPicker/Navbar"));

var _Footer = _interopRequireDefault(require("./EmojiPicker/Footer"));

var _Scroll = _interopRequireDefault(require("./EmojiPicker/Scroll"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

/* eslint-disable */
// Use state reducer to avoid separate re-renders to changes to searchEmojis and focusedEmoji.
function EmojiPickerReducer({
  emojiData
}) {
  return (prevState, nextState) => {
    // reset focusedEmoji to first emoji on searchEmojis change
    if (nextState.searchEmojis && prevState.searchEmojis != nextState.searchEmojis) {
      var _nextState$searchEmoj, _nextState$searchEmoj2, _document$activeEleme;

      let emojis = ((_nextState$searchEmoj = nextState.searchEmojis) === null || _nextState$searchEmoj === void 0 ? void 0 : _nextState$searchEmoj.query) && ((_nextState$searchEmoj2 = nextState.searchEmojis) === null || _nextState$searchEmoj2 === void 0 ? void 0 : _nextState$searchEmoj2.emojis) || emojiData,
          category = emojis[Object.keys(emojis)[0]];
      let emoji = category[0];
      nextState.focusedEmoji = {
        row: 1,
        emoji,
        focusOnRender: Boolean((_document$activeEleme = document.activeElement) === null || _document$activeEleme === void 0 ? void 0 : _document$activeEleme.closest(".emoji-picker-scroll")),
        preventScroll: false
      };
    }

    return { ...prevState,
      ...nextState
    };
  };
}

function EmojiPickerRefComponent({
  emojiData = {},
  emojiSize = 36,
  numberScrollRows = 12,
  onEmojiSelect = emoji => console.log(emoji),
  showNavbar = false,
  showFooter = false,
  showScroll = true,
  emojisPerRow = 9,
  onKeyDownScroll = event => null,
  collapseCategoriesOnSearch = true,
  collapseHeightOnSearch = true,
  theme = "system",
  emojiPreviewName = emoji => emoji.name
}, ref) {
  var _document$activeEleme2, _pickerState$focusedE3;

  const pickerStateReducer = (0, _react.useCallback)(EmojiPickerReducer({
    emojiData
  }), [emojiData]);
  const [pickerState, setPickerState] = (0, _react.useReducer)(pickerStateReducer, {
    searchEmojis: {
      emojis: null,
      query: ""
    },
    focusedEmoji: {
      row: 1,
      emoji: Object.values(emojiData).flat()[0],
      focusOnRender: Boolean((_document$activeEleme2 = document.activeElement) === null || _document$activeEleme2 === void 0 ? void 0 : _document$activeEleme2.closest(".emoji-picker-scroll")),
      preventScroll: false
    }
  });
  const {
    itemCount,
    itemRanges
  } = (0, _react.useMemo)(() => (0, _utils.calcCountAndRange)(pickerState.searchEmojis.emojis || emojiData, emojisPerRow), [pickerState.searchEmojis, emojisPerRow]);
  /**
   * TODO: Replace in-memory search with indexeddb index.
   * See as an example: https://github.com/nolanlawson/emoji-picker-element
   */

  const search = query => {
    var _document$activeEleme3;

    const {
      searchEmojis
    } = pickerState; // reset pickerState when query is empty string

    if (!query) return setPickerState({
      searchEmojis: {
        emojis: null,
        query: ""
      },
      focusedEmoji: {
        row: 1,
        emoji: Object.values(emojiData).flat()[0],
        focusOnRender: Boolean((_document$activeEleme3 = document.activeElement) === null || _document$activeEleme3 === void 0 ? void 0 : _document$activeEleme3.closest(".emoji-picker-scroll")),
        preventScroll: false
      }
    }); // assumption: increasing query length if prevSearchEmojis is empty will not change searchEmojis

    if (searchEmojis !== null && searchEmojis !== void 0 && searchEmojis.emojis && !Object.values(searchEmojis.emojis).flat().length && searchEmojis.query.length < query.length) return setPickerState({
      searchEmojis: {
        emojis: searchEmojis.emojis,
        query
      }
    }); // use prevSearchEmojis if query length has increased, else use full set

    const index = query.length > searchEmojis.query.length && searchEmojis.emojis != null ? Object.values(searchEmojis.emojis).flat() : Object.values(emojiData).flat(); // simple weighted search to filter emojiObjects

    let results = index.map(emoji => ({
      emoji,
      score: (emoji.keywords || []).map(word => word.indexOf(query) != -1).reduce((a, b) => a + Number(b), Number(emoji.name.indexOf(query) != -1) * 3)
    })).filter(a => a.score).sort((a, b) => b.score - a.score).map(({
      emoji
    }) => emoji);

    if (collapseCategoriesOnSearch) {
      return setPickerState({
        searchEmojis: {
          emojis: {
            "Search Results": results
          },
          query
        }
      });
    } else {
      let grouped = Object.entries(emojiData).map(([category, list]) => [category, list.filter(emoji => results.includes(emoji))]).reduce((sum, [category, list]) => Object.assign(sum, {
        [category]: list
      }), {});
      return setPickerState({
        searchEmojis: {
          emojis: grouped,
          query
        }
      });
    }
  };

  const refVirtualList = (0, _react.useRef)(null);
  const refScroll = (0, _react.useRef)(null); // Define event handlers in scroll element.

  const handleClickInScroll = (emoji, row) => event => {
    event.preventDefault(); // MDN docs: keep the focus from leaving the HTMLElement

    onEmojiSelect(emoji, event);
    setPickerState({
      focusedEmoji: {
        row,
        emoji,
        focusOnRender: true,
        preventScroll: true
      }
    });
  };

  const handleMouseInScroll = (emoji, row) => event => {
    var _pickerState$focusedE;

    if (emoji == ((_pickerState$focusedE = pickerState.focusedEmoji) === null || _pickerState$focusedE === void 0 ? void 0 : _pickerState$focusedE.emoji) || event.movementX == 0 && event.movementY == 0) return;
    event.preventDefault(); // MDN docs: keep the focus from leaving the HTMLElement
    // @ts-ignore

    const isSafari = window.safari !== undefined; // safari does not support preventScroll focus

    setPickerState({
      focusedEmoji: {
        row,
        emoji,
        focusOnRender: !isSafari,
        preventScroll: true
      }
    });
    isSafari && refScroll.current && refScroll.current.focus();
  };

  const handleKeyDownScroll = event => {
    const {
      searchEmojis,
      focusedEmoji
    } = pickerState;
    const emojis = Object.values(searchEmojis.emojis || emojiData).filter(array => array.length !== 0);

    switch (event.key) {
      case "Enter":
        {
          event.preventDefault();

          if (!focusedEmoji) {
            var _document$activeEleme4;

            let emoji = Object.values(searchEmojis.emojis || emojiData).flat()[0];
            emoji && setPickerState({
              focusedEmoji: {
                row: 1,
                emoji,
                focusOnRender: Boolean((_document$activeEleme4 = document.activeElement) === null || _document$activeEleme4 === void 0 ? void 0 : _document$activeEleme4.closest(".emoji-picker-scroll")),
                preventScroll: false
              }
            });
          } else {
            focusedEmoji.emoji && onEmojiSelect(focusedEmoji.emoji, event);
          }

          return;
        }

      case 'Home':
        {
          var _document$activeEleme5;

          event.preventDefault();
          let emoji = undefined,
              row = undefined;
          let emojis = searchEmojis.emojis || emojiData,
              category = emojis[Object.keys(emojis)[0]];
          emoji = category[0];
          row = 1;
          return row && emoji && setPickerState({
            focusedEmoji: {
              row,
              emoji,
              focusOnRender: Boolean((_document$activeEleme5 = document.activeElement) === null || _document$activeEleme5 === void 0 ? void 0 : _document$activeEleme5.closest(".emoji-picker-scroll")),
              preventScroll: false
            }
          });
        }

      case 'End':
        {
          var _itemRanges, _document$activeEleme6;

          event.preventDefault();
          let emoji = undefined,
              row = undefined;
          let emojis = searchEmojis.emojis || emojiData,
              category = emojis[Object.keys(emojis).pop()];
          emoji = category[category.length - 1];
          row = ((_itemRanges = itemRanges[itemRanges.length - 1]) === null || _itemRanges === void 0 ? void 0 : _itemRanges.to) - 1;
          return row && emoji && setPickerState({
            focusedEmoji: {
              row,
              emoji,
              focusOnRender: Boolean((_document$activeEleme6 = document.activeElement) === null || _document$activeEleme6 === void 0 ? void 0 : _document$activeEleme6.closest(".emoji-picker-scroll")),
              preventScroll: false
            }
          });
        }

      case "ArrowUp":
        {
          var _document$activeEleme7;

          event.preventDefault();
          let emoji = undefined,
              row = undefined;

          if (!focusedEmoji) {
            emoji = Object.values(searchEmojis.emojis || emojiData).flat()[0];
            row = 1;
          } else {
            let arrayIndex, arrayEmoji, emojiIndex;
            emojis.find((array, index) => {
              emojiIndex = array.findIndex(emoji => emoji === focusedEmoji.emoji), arrayIndex = index, arrayEmoji = array;
              return emojiIndex !== -1;
            });

            if (emojiIndex != undefined) {
              if (emojiIndex - emojisPerRow >= 0) {
                // not first row
                emoji = arrayEmoji[emojiIndex - emojisPerRow];
                row = focusedEmoji.row - 1;
              } else if (arrayIndex !== 0) {
                const arrayAbove = emojis[arrayIndex - 1];
                const index = emojiIndex > (arrayAbove.length - 1) % emojisPerRow ? arrayAbove.length - 1 : Math.floor((arrayAbove.length - 1 - emojiIndex) / emojisPerRow) * emojisPerRow + emojiIndex;
                emoji = arrayAbove[index]; // go directly up if possible; else last element

                row = focusedEmoji.row - 2; // skip category title row
              }
            }
          }

          return row && emoji && setPickerState({
            focusedEmoji: {
              row,
              emoji,
              focusOnRender: Boolean((_document$activeEleme7 = document.activeElement) === null || _document$activeEleme7 === void 0 ? void 0 : _document$activeEleme7.closest(".emoji-picker-scroll")),
              preventScroll: false
            }
          });
        }

      case "ArrowDown":
        {
          var _document$activeEleme8;

          event.preventDefault();
          let emoji = undefined,
              row = undefined;

          if (!focusedEmoji) {
            emoji = Object.values(searchEmojis.emojis || emojiData).flat()[0];
            row = 1;
          } else {
            let arrayIndex, arrayEmoji, emojiIndex;
            emojis.find((array, index) => {
              emojiIndex = array.findIndex(emoji => emoji === focusedEmoji.emoji), arrayIndex = index, arrayEmoji = array;
              return emojiIndex !== -1;
            });

            if (emojiIndex != undefined) {
              if (emojiIndex + emojisPerRow < arrayEmoji.length) {
                // not last row
                emoji = arrayEmoji[emojiIndex + emojisPerRow];
                row = focusedEmoji.row + 1;
              } else if (emojiIndex + emojisPerRow < Math.ceil(arrayEmoji.length / emojisPerRow) * emojisPerRow) {
                emoji = arrayEmoji[arrayEmoji.length - 1];
                row = focusedEmoji.row + 1;
              } else if (arrayIndex !== emojis.length - 1) {
                const arrayBelow = emojis[arrayIndex + 1],
                      modIndex = emojiIndex % emojisPerRow;
                emoji = arrayBelow[modIndex] || arrayBelow[arrayBelow.length - 1]; // go directly down if possible

                row = focusedEmoji.row + 2; // skip category title row
              }
            }
          }

          return row && emoji && setPickerState({
            focusedEmoji: {
              row,
              emoji,
              focusOnRender: Boolean((_document$activeEleme8 = document.activeElement) === null || _document$activeEleme8 === void 0 ? void 0 : _document$activeEleme8.closest(".emoji-picker-scroll")),
              preventScroll: false
            }
          });
        }

      case "ArrowLeft":
        {
          var _document$activeEleme9;

          event.preventDefault();
          let emoji = undefined,
              row = undefined;

          if (!focusedEmoji) {
            emoji = Object.values(searchEmojis.emojis || emojiData).flat()[0];
            row = 1;
          } else {
            let arrayIndex, arrayEmoji, emojiIndex;
            emojis.find((array, index) => {
              emojiIndex = array.findIndex(emoji => emoji === focusedEmoji.emoji), arrayIndex = index, arrayEmoji = array;
              return emojiIndex !== -1;
            });

            if (emojiIndex != undefined) {
              if (emojiIndex - 1 >= 0) {
                emoji = arrayEmoji[emojiIndex - 1];
                row = Math.floor(emojiIndex / emojisPerRow) == Math.floor((emojiIndex - 1) / emojisPerRow) ? focusedEmoji.row : focusedEmoji.row - 1;
              } else if (arrayIndex !== 0) {
                // category above this one if it exists
                const arrayAbove = emojis[arrayIndex - 1];
                emoji = arrayAbove[arrayAbove.length - 1];
                row = focusedEmoji.row - 2; // skip category title row
              }
            }
          }

          return row && emoji && setPickerState({
            focusedEmoji: {
              row,
              emoji,
              focusOnRender: Boolean((_document$activeEleme9 = document.activeElement) === null || _document$activeEleme9 === void 0 ? void 0 : _document$activeEleme9.closest(".emoji-picker-scroll")),
              preventScroll: false
            }
          });
        }

      case "ArrowRight":
        {
          var _document$activeEleme10;

          event.preventDefault();
          let emoji = undefined,
              row = undefined;

          if (!focusedEmoji) {
            emoji = Object.values(searchEmojis.emojis || emojiData).flat()[0];
            row = 1;
          } else {
            let arrayIndex, arrayEmoji, emojiIndex;
            emojis.find((array, index) => {
              emojiIndex = array.findIndex(emoji => emoji === focusedEmoji.emoji), arrayIndex = index, arrayEmoji = array;
              return emojiIndex !== -1;
            });

            if (emojiIndex != undefined) {
              let newIndex = emojiIndex + 1;

              if (newIndex < arrayEmoji.length) {
                emoji = arrayEmoji[newIndex];
                row = Math.floor(emojiIndex / emojisPerRow) == Math.floor(newIndex / emojisPerRow) ? focusedEmoji.row : focusedEmoji.row + 1;
              } else if (arrayIndex !== emojis.length - 1) {
                let arrayBelow = emojis[arrayIndex + 1];
                emoji = arrayBelow[0];
                row = focusedEmoji.row + 2; // skip category title row
              }
            }
          }

          return row && emoji && setPickerState({
            focusedEmoji: {
              row,
              emoji,
              focusOnRender: Boolean((_document$activeEleme10 = document.activeElement) === null || _document$activeEleme10 === void 0 ? void 0 : _document$activeEleme10.closest(".emoji-picker-scroll")),
              preventScroll: false
            }
          });
        }

      default:
        return onKeyDownScroll(event);
    }
  }; // Make internal state and methods as publicly accessible via ref.


  (0, _react.useImperativeHandle)(ref, () => {
    var _pickerState$focusedE2;

    return {
      search,
      handleKeyDownScroll,
      emojis: pickerState.searchEmojis.emojis || emojiData,
      focusedEmoji: (_pickerState$focusedE2 = pickerState.focusedEmoji) === null || _pickerState$focusedE2 === void 0 ? void 0 : _pickerState$focusedE2.emoji
    };
  });
  const ScrollProps = {
    emojiData: pickerState.searchEmojis.emojis || emojiData,
    emojisPerRow: emojisPerRow,
    emojiSize,
    numberScrollRows,
    focusedEmoji: pickerState.focusedEmoji,
    refVirtualList,
    handleClickInScroll,
    handleMouseInScroll,
    collapseHeightOnSearch,
    itemCount,
    itemRanges
  }; // Define event handler for select in navbar element.

  const handleSelectInNavbar = category => {
    let virtualList = refVirtualList.current;

    if (virtualList) {
      let range = itemRanges.find(range => range.key === category);

      if (range) {
        setPickerState({
          focusedEmoji: {
            row: range.from + 1,
            emoji: emojiData[category][0],
            focusOnRender: false,
            preventScroll: false
          }
        });
        virtualList.scrollToItem(range.from, "start");
      }
    }
  }; // Compute width on window resize.


  const [width, setWidth] = (0, _react.useState)(() => `calc(${emojiSize}px * ${emojisPerRow} + 1em + ${(0, _utils.measureScrollbar)()}px)`);
  (0, _react.useLayoutEffect)(() => {
    let resizeWidth = () => setWidth(`calc(${emojiSize}px * ${emojisPerRow} + 1em + ${(0, _utils.measureScrollbar)()}px)`);

    window.addEventListener("resize", resizeWidth);
    return () => window.removeEventListener("resize", resizeWidth);
  }, []);
  return /*#__PURE__*/_react.default.createElement("div", {
    className: `emoji-picker emoji-picker-${theme}`,
    style: {
      width
    }
  }, showNavbar && /*#__PURE__*/_react.default.createElement(_Navbar.default, {
    data: emojiData,
    handleSelectInNavbar: handleSelectInNavbar
  }), /*#__PURE__*/_react.default.createElement("div", {
    className: "emoji-picker-scroll",
    role: "grid",
    "aria-rowcount": itemCount,
    "aria-colcount": emojisPerRow,
    onKeyDown: handleKeyDownScroll,
    ref: refScroll
  }, pickerState.searchEmojis.emojis ? Object.values(pickerState.searchEmojis.emojis).flat().length ? /*#__PURE__*/_react.default.createElement(_Scroll.default, ScrollProps) : /*#__PURE__*/_react.default.createElement("div", {
    className: "emoji-picker-category",
    style: {
      height: collapseHeightOnSearch ? 'inherit' : '432px'
    }
  }, /*#__PURE__*/_react.default.createElement("div", {
    className: "emoji-picker-category-title"
  }, "No results")) : showScroll && /*#__PURE__*/_react.default.createElement(_Scroll.default, ScrollProps)), showFooter && /*#__PURE__*/_react.default.createElement(_Footer.default, {
    emoji: (_pickerState$focusedE3 = pickerState.focusedEmoji) === null || _pickerState$focusedE3 === void 0 ? void 0 : _pickerState$focusedE3.emoji,
    emojiPreviewName: emojiPreviewName
  }));
}

const EmojiPicker = /*#__PURE__*/(0, _react.forwardRef)(EmojiPickerRefComponent);
exports.EmojiPicker = EmojiPicker;