"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _react = _interopRequireWildcard(require("react"));

var _reactWindow = require("react-window");

var _reactWindowInfiniteLoader = _interopRequireDefault(require("react-window-infinite-loader"));

var _utils = require("../utils");

var _Emoji = _interopRequireDefault(require("../Emoji"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _getRequireWildcardCache(nodeInterop) { if (typeof WeakMap !== "function") return null; var cacheBabelInterop = new WeakMap(); var cacheNodeInterop = new WeakMap(); return (_getRequireWildcardCache = function (nodeInterop) { return nodeInterop ? cacheNodeInterop : cacheBabelInterop; })(nodeInterop); }

function _interopRequireWildcard(obj, nodeInterop) { if (!nodeInterop && obj && obj.__esModule) { return obj; } if (obj === null || typeof obj !== "object" && typeof obj !== "function") { return { default: obj }; } var cache = _getRequireWildcardCache(nodeInterop); if (cache && cache.has(obj)) { return cache.get(obj); } var newObj = {}; var hasPropertyDescriptor = Object.defineProperty && Object.getOwnPropertyDescriptor; for (var key in obj) { if (key !== "default" && Object.prototype.hasOwnProperty.call(obj, key)) { var desc = hasPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : null; if (desc && (desc.get || desc.set)) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } newObj.default = obj; if (cache) { cache.set(obj, newObj); } return newObj; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

const Scroll = ({
  emojisPerRow,
  emojiSize,
  numberScrollRows,
  focusedEmoji,
  emojiData,
  refVirtualList,
  handleClickInScroll,
  handleMouseInScroll,
  itemCount,
  itemRanges,
  collapseHeightOnSearch
}) => {
  const [arrayOfRows, setArrayOfRows] = (0, _react.useState)({});
  const infiniteLoaderRef = (0, _react.useRef)(null); // Keep track of previously focused emoji to avoid re-rendering all rows.

  const prevFocusedEmoji = (0, _react.useRef)(null); // Reset arrayOfRows upon change in data or emojisPerRow.

  (0, _react.useEffect)(function resetScrollState() {
    setArrayOfRows({});
    infiniteLoaderRef === null || infiniteLoaderRef === void 0 ? void 0 : infiniteLoaderRef.current.resetloadMoreItemsCache();
    prevFocusedEmoji.current = focusedEmoji; // focusedEmoji included in emojiData change render

    refVirtualList === null || refVirtualList === void 0 ? void 0 : refVirtualList.current.scrollToItem(0);
    loadMoreItems(0, Math.min(numberScrollRows + 10 - 1, itemRanges[itemRanges.length - 1].to)); // minimumBatchSize + threshold - 1
  }, [emojiData, emojisPerRow]); // Recompute the rows of the next and previous focusedEmoji upon change in focusedEmoji.

  (0, _react.useEffect)(function resetRowsWithFocusedEmoji() {
    var _refVirtualList$curre;

    let prevEmoji = prevFocusedEmoji.current,
        nextEmoji = focusedEmoji;

    if (prevEmoji == nextEmoji) {
      return;
    }

    let rowsToUpdate = new Set([prevEmoji === null || prevEmoji === void 0 ? void 0 : prevEmoji.row, nextEmoji === null || nextEmoji === void 0 ? void 0 : nextEmoji.row]);
    Array.from(rowsToUpdate).forEach(row => row && loadMoreItems(row, row));
    prevFocusedEmoji.current = nextEmoji;
    (nextEmoji === null || nextEmoji === void 0 ? void 0 : nextEmoji.row) && ((_refVirtualList$curre = refVirtualList.current) === null || _refVirtualList$curre === void 0 ? void 0 : _refVirtualList$curre.scrollToItem(nextEmoji.row));
  }, [focusedEmoji]);

  const isItemLoaded = index => !!arrayOfRows[index];

  const loadMoreItems = (startIndex, endIndex) => {
    const nextArrayOfRows = {};
    let i = startIndex,
        range;

    while (i <= endIndex) {
      range = itemRanges.find(range => range.from <= i && i < range.to);
      if (range === undefined) break;

      for (let rowIndex = i; rowIndex < Math.min(range.to, endIndex + 1); rowIndex++) {
        if (rowIndex == range.from) {
          nextArrayOfRows[rowIndex] = /*#__PURE__*/_react.default.createElement("div", {
            className: "emoji-picker-category-title",
            "aria-rowindex": rowIndex + 1,
            "aria-colindex": 1
          }, range.key);
        } else {
          const offset = rowIndex - range.from;
          const row = emojiData[range.key].slice((offset - 1) * emojisPerRow, offset * emojisPerRow);
          nextArrayOfRows[rowIndex] = /*#__PURE__*/_react.default.createElement("ul", {
            className: "emoji-picker-category-emoji",
            role: "row",
            "aria-rowindex": rowIndex + 1
          }, row.map((emoji, colIndex) => {
            const liProps = {
              key: emoji.unicode,
              // onClick: handleClickInScroll(emoji, rowIndex),
              onMouseDown: ev => {
                handleClickInScroll(emoji, rowIndex)(ev);
                ev.nativeEvent.stopImmediatePropagation();
              },
              onMouseMove: handleMouseInScroll(emoji, rowIndex),
              role: "gridcell",
              "aria-rowindex": rowIndex + 1,
              "aria-colindex": colIndex + 1,
              tabIndex: -1,
              ...(emoji === (focusedEmoji === null || focusedEmoji === void 0 ? void 0 : focusedEmoji.emoji) && {
                tabIndex: 0,
                ref: li => focusedEmoji.focusOnRender && (li === null || li === void 0 ? void 0 : li.focus({
                  preventScroll: focusedEmoji.preventScroll
                }))
              })
            };
            const emojiProps = {
              emoji,
              ...(emoji === (focusedEmoji === null || focusedEmoji === void 0 ? void 0 : focusedEmoji.emoji) && {
                className: "emoji-picker-emoji-focused"
              })
            };
            return /*#__PURE__*/_react.default.createElement("li", liProps, /*#__PURE__*/_react.default.createElement(_Emoji.default, emojiProps));
          }));
        }
      }

      i = range.to;
    }

    setArrayOfRows(prev => Object.assign({}, prev, nextArrayOfRows));
  };

  return /*#__PURE__*/_react.default.createElement(_reactWindowInfiniteLoader.default, {
    ref: infiniteLoaderRef,
    itemCount: itemCount,
    loadMoreItems: loadMoreItems,
    isItemLoaded: isItemLoaded,
    minimumBatchSize: numberScrollRows,
    threshold: 10
  }, ({
    onItemsRendered,
    ref
  }) => /*#__PURE__*/_react.default.createElement(_reactWindow.FixedSizeList, {
    onItemsRendered: onItemsRendered,
    ref: list => {
      typeof ref === 'function' && ref(list);
      refVirtualList && (refVirtualList.current = list);
    },
    itemCount: itemCount,
    itemData: arrayOfRows,
    itemSize: emojiSize,
    height: collapseHeightOnSearch ? Math.min(itemCount * emojiSize + 9, numberScrollRows * emojiSize) : numberScrollRows * emojiSize,
    innerElementType: innerElementType,
    width: "unset"
  }, MemoizedRow));
};

const MemoizedScroll = /*#__PURE__*/(0, _react.memo)(Scroll, function ScrollPropsAreEqual(prevProps, nextProps) {
  var _prevProps$focusedEmo, _nextProps$focusedEmo;

  return ((_prevProps$focusedEmo = prevProps.focusedEmoji) === null || _prevProps$focusedEmo === void 0 ? void 0 : _prevProps$focusedEmo.emoji) == ((_nextProps$focusedEmo = nextProps.focusedEmoji) === null || _nextProps$focusedEmo === void 0 ? void 0 : _nextProps$focusedEmo.emoji) && prevProps.emojiData == nextProps.emojiData && prevProps.collapseHeightOnSearch == nextProps.collapseHeightOnSearch && prevProps.emojiSize == nextProps.emojiSize && prevProps.emojisPerRow == nextProps.emojisPerRow;
});
var _default = MemoizedScroll;
exports.default = _default;

const VirtualRow = ({
  index,
  style,
  data
}) => {
  return /*#__PURE__*/_react.default.createElement("div", {
    className: "emoji-picker-virtual-row",
    style: style
  }, data[index]);
};

/**
 * memoize rows of the virtualList, only re-rendering when changing in data[index]
 */
const MemoizedRow = /*#__PURE__*/(0, _react.memo)(VirtualRow, function compareRowProps(prevProps, nextProps) {
  const {
    style: prevStyle,
    data: prevData,
    index: prevIndex,
    ...prevRest
  } = prevProps;
  const {
    style: nextStyle,
    data: nextData,
    index: nextIndex,
    ...nextRest
  } = nextProps;
  return prevData[prevIndex] === nextData[nextIndex] && !(0, _utils.shallowDiffer)(prevStyle, nextStyle) && !(0, _utils.shallowDiffer)(prevRest, nextRest);
});
/**
 * adds padding to the bottom of virtual list
 * See: https://github.com/bvaughn/react-window#can-i-add-padding-to-the-top-and-bottom-of-a-list
 */

const LIST_PADDING_SIZE = 9;
const innerElementType = /*#__PURE__*/(0, _react.forwardRef)(({
  style,
  ...props
}, ref) =>
/*#__PURE__*/
// @ts-ignore
_react.default.createElement("div", _extends({
  ref: ref,
  style: { ...style,
    height: `${parseFloat(style.height) + LIST_PADDING_SIZE}px`
  }
}, props)));